package net.sf.mgp.unclasses;

/*
 * Copyright 2008 MGP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodAdapter;
import org.objectweb.asm.MethodVisitor;

/**
 * Implementation of {@link DependencyResolver} resolving class annotation dependencies. Implemented using ObjectWeb ASM (http://asm.objectweb.org/).
 * 
 * <p>
 * This class should be removed as soon as released version of Apache BCEL will resolve annotation dependencies.
 * </p>
 * <p>
 * Please note this implementation do not resolve dependencies on Local Variable annotations (http://mail-archive.ow2.org/asm/2005-06/msg00041.html).
 * </p>
 * 
 * @plexus.component role="net.sf.mgp.unclasses.DependencyResolver" role-hint="annotations"
 * 
 * @author Maksim Gordienko
 * 
 */
public class AsmClassAnnotationResolver extends AbstractAsmDependencyResolver {

	protected Collection getDependencyNames(ClassReader reader) {
		final List annotations = new ArrayList();
		reader.accept(new ClassAdapter(createEmptyClassVisitor()) {
			private final FieldVisitor fieldVisitor = new FieldVisitor() {
				public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
					addAnnotation(desc);
					return null;
				}

				public void visitAttribute(Attribute attr) {
				}

				public void visitEnd() {
				}
			};

			private final MethodVisitor methodVisitor = new MethodAdapter(createEmptyMethodVisitor()) {
				public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
					addAnnotation(desc);
					return null;
				}

				public AnnotationVisitor visitParameterAnnotation(int parameter, String desc, boolean visible) {
					addAnnotation(desc);
					return null;
				}
			};

			protected void addAnnotation(String desc) {
				annotations.add(Utils.getClassNameFromDescriptor(desc));
			}

			public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
				addAnnotation(desc);
				return null;
			}

			public FieldVisitor visitField(int access, String name, String desc, String signature, Object value) {
				return fieldVisitor;
			}

			public MethodVisitor visitMethod(int access, String name, String desc, String signature, String[] exceptions) {
				return methodVisitor;
			}

		}, 0);
		return annotations;
	}
}
