package net.sf.mgp.unclasses;

/*
 * Copyright 2008 MGP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

class Utils {
	private Utils() {
	}

	public static final String FILE_SUFFIX = ".class";

	public static String getClassName(String classFile) {
		String className = classFile.replace(File.separatorChar, '.');
		className = className.substring(0, className.length() - FILE_SUFFIX.length());
		return className;
	}

	private static final Pattern REF_NAME_REGEXP = Pattern.compile("\\[*L(.*);");

	public static String getClassNameFromDescriptor(String descriptor) {
		Matcher m = REF_NAME_REGEXP.matcher(descriptor);
		if (!m.matches()) {
			throw new IllegalArgumentException("Invalid class descriptor " + descriptor);
		}
		return m.group(1).replace('/', '.');
	}

	public static boolean isClassDescriptor(String descriptor) {
		return REF_NAME_REGEXP.matcher(descriptor).matches();
	}
}
